@@ -1,20 +1,23 @@
 /////////////////////////////////////////////////////////////////////////////
 // Name:        treelistctrl.h
 // Purpose:     wxTreeListCtrl class
-// Author:      Robert Roebling
-// Maintainer:  Otto Wyss
 // Created:     01/02/97
-// Copyright:   (c) 2004 Robert Roebling, Julian Smart, Alberto Griggio,
-//              Vadim Zeitlin, Otto Wyss
-// Licence:     wxWindows license
-/////////////////////////////////////////////////////////////////////////////
-// Additions for wxPerl copyright (c) 2010 Mark Dootson
-// SVN ID:      $Id: treelistctrl.h 3 2010-02-17 06:08:51Z mark.dootson $
+// Author:      Robert Roebling
+// Maintainer:  Ronan Chartois (pgriddev)
+// Version:     $Id: treelistctrl.h 17 2011-06-21 14:21:11Z mark.dootson $
+// Copyright:   (c) 2004-2011 Robert Roebling, Julian Smart, Alberto Griggio,
+//              Vadim Zeitlin, Otto Wyss, Ronan Chartois
+// Licence:     wxWindows
 /////////////////////////////////////////////////////////////////////////////
 
+
 #ifndef TREELISTCTRL_H
 #define TREELISTCTRL_H
 
+#if defined(__GNUG__) && !defined(__APPLE__)
+    #pragma interface "treelistctrl.h"
+#endif
+
 #include <wx/treectrl.h>
 #include <wx/control.h>
 #include <wx/pen.h>
@@ -26,13 +29,12 @@
 #define GIZMODLLEXPORT
 #endif
 
-
 class GIZMODLLEXPORT wxTreeListItem;
 class GIZMODLLEXPORT wxTreeListHeaderWindow;
 class GIZMODLLEXPORT wxTreeListMainWindow;
 
-#define wxTR_COLUMN_LINES            0x1000 // put border around items
-#define wxTR_VIRTUAL                 0x4000 // The application provides items text on demand.
+#define wxTR_COLUMN_LINES 0x1000 // put border around items
+#define wxTR_VIRTUAL      0x4000 // The application provides items text on demand.
 #define wxTR_SHOW_ROOT_LABEL_ONLY    0x8000
 
 // Using this typedef removes an ambiguity when calling Remove()
@@ -271,7 +273,6 @@ wxTreeListColumnInfoInternal& wxTreeListColumnInfo::GetInternalInfoObject() cons
 }
 
 
-
 //----------------------------------------------------------------------------
 // wxTreeListCtrl - the multicolumn tree control
 //----------------------------------------------------------------------------
@@ -294,9 +295,13 @@ extern GIZMODLLEXPORT const wxChar* wxTreeListCtrlNameStr;
 
 class GIZMODLLEXPORT wxTreeListCtrl : public wxControl
 {
+friend class wxTreeListHeaderWindow;
+friend class wxTreeListMainWindow;
+friend class wxTreeListItem;
 public:
-    // creation
-    // --------
+
+    // ---------- creation ----------
+
     wxTreeListCtrl()
         : m_header_win(0), m_main_win(0), m_headerHeight(0)
     {}
@@ -323,8 +328,9 @@ public:
 
     void Refresh(bool erase=TRUE, const wxRect* rect=NULL);
     void SetFocus();
-    // accessors
-    // ---------
+
+
+    // ---------- general methods ----------
 
     // get the total number of items in the control
     size_t GetCount() const;
@@ -362,8 +368,11 @@ public:
     void AssignStateImageList(wxImageList *imageList);
     void AssignButtonsImageList(wxImageList *imageList);
 
+    void SetToolTip(const wxString& tip);
+    void SetToolTip (wxToolTip *tip);
+    void SetItemToolTip(const wxTreeItemId& item, const wxString &tip);
 
-    // Functions to work with columns
+    // ---------- Functions to work with columns ----------
 
     // adds a column
     void AddColumn (const wxString& text,
@@ -421,49 +430,57 @@ public:
     void SetColumnEditable (int column, bool edit = true);
     bool IsColumnEditable (int column) const;
 
-    // Functions to work with items.
+    // ----------  Functions to work with items. ----------
 
-    // accessors
+    // accessors (most properties have a default at row/item level)
     // ---------
 
-    // retrieve item's label (of the main column)
-    wxString GetItemText (const wxTreeItemId& item) const
-        { return GetItemText (item, GetMainColumn()); }
-    // retrieves item's label of the given column
+    wxString GetItemText (const wxTreeItemId& item)             const { return GetItemText (item, GetMainColumn()); };
     wxString GetItemText (const wxTreeItemId& item, int column) const;
 
-    // get one of the images associated with the item (normal by default)
-    int GetItemImage (const wxTreeItemId& item,
-                      wxTreeItemIcon which = wxTreeItemIcon_Normal) const
-    { return GetItemImage (item, GetMainColumn(), which); }
-    int GetItemImage (const wxTreeItemId& item, int column,
-                      wxTreeItemIcon which = wxTreeItemIcon_Normal) const;
+    int GetItemImage (const wxTreeItemId& item, wxTreeItemIcon which = wxTreeItemIcon_Normal) const;
+    int GetItemImage (const wxTreeItemId& item, int column) const;
 
-    // get the data associated with the item
-    wxTreeItemData *GetItemData (const wxTreeItemId& item) const;
+    wxTreeItemData *GetItemData (const wxTreeItemId& item)             const;
+    wxTreeItemData *GetItemData (const wxTreeItemId& item, int column) const;
 
-    bool GetItemBold (const wxTreeItemId& item) const;
-    wxColour GetItemTextColour (const wxTreeItemId& item) const;
-    wxColour GetItemBackgroundColour (const wxTreeItemId& item) const;
-    wxFont GetItemFont (const wxTreeItemId& item) const;
+    bool GetItemBold (const wxTreeItemId& item)             const;
+    bool GetItemBold (const wxTreeItemId& item, int column) const;
 
-    // modifiers
+    wxColour GetItemTextColour (const wxTreeItemId& item)             const;
+    wxColour GetItemTextColour (const wxTreeItemId& item, int column) const;
 
-    // set item's label
-    void SetItemText (const wxTreeItemId& item, const wxString& text)
-        { SetItemText (item, GetMainColumn(), text); }
+    wxColour GetItemBackgroundColour (const wxTreeItemId& item)             const;
+    wxColour GetItemBackgroundColour (const wxTreeItemId& item, int column) const;
+
+    wxFont GetItemFont (const wxTreeItemId& item)             const;
+    wxFont GetItemFont (const wxTreeItemId& item, int column) const;
+
+    // modifiers (most properties have a default at row/item level)
+    // ---------
+
+    void SetItemText (const wxTreeItemId& item, const wxString& text) { }
     void SetItemText (const wxTreeItemId& item, int column, const wxString& text);
 
-    // get one of the images associated with the item (normal by default)
-    void SetItemImage (const wxTreeItemId& item, int image,
-                       wxTreeItemIcon which = wxTreeItemIcon_Normal)
-        { SetItemImage (item, GetMainColumn(), image, which); }
     // the which parameter is ignored for all columns but the main one
-    void SetItemImage (const wxTreeItemId& item, int column, int image,
-                       wxTreeItemIcon which = wxTreeItemIcon_Normal);
+    void SetItemImage (const wxTreeItemId& item, int image, wxTreeItemIcon which = wxTreeItemIcon_Normal);
+    void SetItemImage (const wxTreeItemId& item, int column, int image);
 
-    // associate some data with the item
-    void SetItemData (const wxTreeItemId& item, wxTreeItemData *data);
+    void SetItemData (const wxTreeItemId& item,             wxTreeItemData *data);
+    void SetItemData (const wxTreeItemId& item, int column, wxTreeItemData *data);
+
+    void SetItemBold (const wxTreeItemId& item,             bool bold = true);
+    void SetItemBold (const wxTreeItemId& item, int column, bool bold = true);
+
+    void SetItemTextColour (const wxTreeItemId& item,             const wxColour& colour);
+    void SetItemTextColour (const wxTreeItemId& item, int column, const wxColour& colour);
+
+    void SetItemBackgroundColour (const wxTreeItemId& item,             const wxColour& colour);
+    void SetItemBackgroundColour (const wxTreeItemId& item, int column, const wxColour& colour);
+
+    // font should be of the same height for all items
+    void SetItemFont (const wxTreeItemId& item,             const wxFont& font);
+    void SetItemFont (const wxTreeItemId& item, int column, const wxFont& font);
 
     // force appearance of [+] button near the item. This is useful to
     // allow the user to expand the items which don't have any children now
@@ -471,17 +488,22 @@ public:
     // usage and loading time.
     void SetItemHasChildren(const wxTreeItemId& item, bool has = true);
 
-    // the item will be shown in bold
-    void SetItemBold (const wxTreeItemId& item, bool bold = true);
+    // item status inquiries
+    // ---------------------
 
-    // set the item's text colour
-    void SetItemTextColour (const wxTreeItemId& item, const wxColour& colour);
+    // is the item visible (it might be outside the view or not expanded)?
+    bool IsVisible (const wxTreeItemId& item, bool fullRow = false, bool within = true) const;
+    // does the item has any children?
+    bool HasChildren (const wxTreeItemId& item) const;
+    // is the item expanded (only makes sense if HasChildren())?
+    bool IsExpanded (const wxTreeItemId& item) const;
+    // is this item currently selected (the same as has focus)?
+    bool IsSelected (const wxTreeItemId& item) const;
+    // is item text in bold font?
+    bool IsBold (const wxTreeItemId& item)             const { return IsBold(item, GetMainColumn()); };
+    bool IsBold (const wxTreeItemId& item, int column) const { return GetItemBold(item, column); };
+    // does the layout include space for a button?
 
-    // set the item's background colour
-    void SetItemBackgroundColour (const wxTreeItemId& item, const wxColour& colour);
-
-    // set the item's font (should be of the same height for all items)
-    void SetItemFont (const wxTreeItemId& item, const wxFont& font);
 
     // set the window font
     virtual bool SetFont ( const wxFont &font );
@@ -491,21 +513,6 @@ public:
     long GetWindowStyle() const;
     long GetWindowStyleFlag () const { return GetWindowStyle(); }
 
-    // item status inquiries
-    // ---------------------
-
-    // is the item visible (it might be outside the view or not expanded)?
-    bool IsVisible (const wxTreeItemId& item, bool fullRow = false) const;
-    // does the item has any children?
-    bool HasChildren (const wxTreeItemId& item) const;
-    // is the item expanded (only makes sense if HasChildren())?
-    bool IsExpanded (const wxTreeItemId& item) const;
-    // is this item currently selected (the same as has focus)?
-    bool IsSelected (const wxTreeItemId& item) const;
-    // is item text in bold font?
-    bool IsBold (const wxTreeItemId& item) const;
-    // does the layout include space for a button?
-
     // number of children
     // ------------------
 
@@ -530,9 +537,6 @@ public:
     // get the parent of this item (may return NULL if root)
     wxTreeItemId GetItemParent (const wxTreeItemId& item) const;
 
-    wxTreeItemId GetCurrentItem() const;
-    void SetCurrentItem(const wxTreeItemId& newItem);
-
     // for this enumeration function you must pass in a "cookie" parameter
     // which is opaque for the application but is necessary for the library
     // to make these functions reentrant (i.e. allow more than one
@@ -567,9 +571,11 @@ public:
     wxTreeItemId GetPrevExpanded(const wxTreeItemId& item) const;
 
     // get visible item, see IsVisible()
-    wxTreeItemId GetFirstVisibleItem(bool fullRow = false) const;
-    wxTreeItemId GetNextVisible(const wxTreeItemId& item, bool fullRow = false) const;
-    wxTreeItemId GetPrevVisible(const wxTreeItemId& item, bool fullRow = false) const;
+    wxTreeItemId GetFirstVisibleItem(                      bool fullRow = false) const;
+    wxTreeItemId GetFirstVisible(                          bool fullRow = false, bool within = true) const;
+    wxTreeItemId GetNextVisible (const wxTreeItemId& item, bool fullRow = false, bool within = true) const;
+    wxTreeItemId GetPrevVisible (const wxTreeItemId& item, bool fullRow = false, bool within = true) const;
+    wxTreeItemId GetLastVisible (                          bool fullRow = false, bool within = true) const;
 
     // operations
     // ----------
@@ -605,13 +611,11 @@ public:
                              int image = -1, int selectedImage = -1,
                              wxTreeItemData *data = NULL);
 
-    // delete this item (except root) and associated data if any
+    // delete this item (except root) + children and associated data if any
     void Delete (const wxTreeItemId& item);
     // delete all children (but don't delete the item itself)
-    // NB: this won't send wxEVT_COMMAND_TREE_ITEM_DELETED events
     void DeleteChildren (const wxTreeItemId& item);
     // delete the root and all its children from the tree
-    // NB: this won't send wxEVT_COMMAND_TREE_ITEM_DELETED events
     void DeleteRoot();
 
     // expand this item
@@ -625,11 +629,14 @@ public:
     // toggles the current state
     void Toggle (const wxTreeItemId& item);
 
+    // set cursor item (indicated by black rectangle)
+    void SetCurrentItem(const wxTreeItemId& item = (wxTreeItemId*)NULL);
+
     // remove the selection from currently selected item (if any)
     void Unselect();
     void UnselectAll();
-    // select this item
-    void SelectItem (const wxTreeItemId& item,
+    // select this item - return true if selection was allowed (no veto)
+    bool SelectItem (const wxTreeItemId& item,
                      const wxTreeItemId& last = (wxTreeItemId*)NULL,
                      bool unselect_others = true);
     // select all items in the expanded tree
@@ -659,6 +666,7 @@ public:
         { EditLabel (item, GetMainColumn()); }
     // edit item's label of the given column
     void EditLabel (const wxTreeItemId& item, int column);
+    void EndEdit(bool isCancelled);
 
     // virtual mode
     virtual wxString OnGetItemText( wxTreeItemData* item, long column ) const;
@@ -669,12 +677,14 @@ public:
     // second one. The base class version performs alphabetic comparaison
     // of item labels (GetText)
     virtual int OnCompareItems (const wxTreeItemId& item1, const wxTreeItemId& item2);
+    virtual int OnCompareItems (const wxTreeItemId& item1, const wxTreeItemId& item2, int column);
     // sort the children of this item using OnCompareItems
-    // NB: this function is not reentrant and not MT-safe (FIXME)!
-    void SortChildren(const wxTreeItemId& item);
+    // NB: this function is not reentrant and not MT-safe (TODO)!
+    void SortChildren(const wxTreeItemId& item, int column = -1, bool reverseOrder = false);
 
-    // searching
-    wxTreeItemId FindItem (const wxTreeItemId& item, const wxString& str, int mode = 0);
+    // searching (by column only)
+    wxTreeItemId FindItem (const wxTreeItemId& item,             const wxString& str, int mode = 0) { return FindItem(item, GetMainColumn(), str, mode); };
+    wxTreeItemId FindItem (const wxTreeItemId& item, int column, const wxString& str, int mode = 0);
 
     // overridden base class virtuals
     virtual bool SetBackgroundColour (const wxColour& colour);
@@ -684,20 +694,20 @@ public:
     void SetDragItem (const wxTreeItemId& item = (wxTreeItemId*)NULL);
 
 
-    wxTreeListHeaderWindow* GetHeaderWindow() const
-        { return m_header_win; }
-
-    wxTreeListMainWindow* GetMainWindow() const
-        { return m_main_win; }
-
     virtual wxSize DoGetBestSize() const;
 
 protected:
     // header window, responsible for column visualization and manipulation
-    wxTreeListHeaderWindow* m_header_win;
+    wxTreeListHeaderWindow* GetHeaderWindow() const
+        { return m_header_win; }
+    wxTreeListHeaderWindow* m_header_win;  // future cleanup: make private or remove GetHeaderWindow()
 
     // main window, the "true" tree ctrl
-    wxTreeListMainWindow* m_main_win;
+    wxTreeListMainWindow* GetMainWindow() const
+        { return m_main_win; }
+    wxTreeListMainWindow* m_main_win;  // future cleanup: make private or remove GetMainWindow()
+
+    int GetHeaderHeight() const { return m_headerHeight; }
 
     void CalculateAndSetHeaderHeight();
     void DoHeaderLayout();
